import os
import shutil

# Script config
skip_test = True
restart_nginx = False
cleanup_backup = False

# Load data from services.py and domains.py
from services import services
from domains import domains

# Define paths and filenames
nginx_config_dir = "./"
sites_enabled_dir = os.path.join(nginx_config_dir, "sites-enabled")
backup_dir = "./backup/"

# Cleanup backup directory
if os.path.exists(backup_dir):
    shutil.rmtree(backup_dir)

os.makedirs(backup_dir)

# 1. Backup /etc/nginx/sites-enabled
if os.path.exists(sites_enabled_dir):
    shutil.copytree(sites_enabled_dir, os.path.join(backup_dir, "sites-enabled"))
else:
    os.makedirs(sites_enabled_dir)

# 2. Delete contents of /etc/nginx/sites-enabled
for file in os.listdir(sites_enabled_dir):
    file_path = os.path.join(sites_enabled_dir, file)
    if os.path.isfile(file_path):
        os.remove(file_path)

# 3. Generate configuration files for each domain
template_file = "upgrade.template"

with open(template_file, "r") as template:
    template_text = template.read()

for domain in domains:
    domain_config = template_text.replace("%domain%", domain)
    domain_config_file = os.path.join(sites_enabled_dir, f"1https.{domain}.conf")
    with open(domain_config_file, "w") as conf_file:
        conf_file.write(domain_config)

# 4. Generate configuration files for each service and domain
service_template_file = "service.template"

with open(service_template_file, "r") as service_template:
    service_template_text = service_template.read()

for service_name, service_port in services.items():
    print(f"Generating config for {service_name} ({service_port})")
    final_service_config = ""
    for domain in domains:
        print(f"- {domain}")
        if service_name == "DEFAULT":
            service_config = service_template_text.replace("%service_name%.%domain%", domain)
        else:
            service_config = service_template_text.replace("%domain%", domain)

        service_config = service_config.replace("%domain%", domain)
        service_config = service_config.replace("%service_name%", service_name)
        service_config = service_config.replace("%service_port%", service_port)
        final_service_config = final_service_config + service_config + "\n\n"

    service_config_file = os.path.join(sites_enabled_dir, f"{service_name}.conf")
    with open(service_config_file, "a") as conf_file:
        conf_file.write(final_service_config)

# 5. Check Nginx configuration
if not skip_test:
    test_result = os.system("nginx -t")

    if test_result == 0:
        print("Nginx configuration test successful. Reloading Nginx...")
        if restart_nginx:
            os.system("nginx -s reload")
    else:
        print("Nginx configuration test failed. Restoring backup...")
        shutil.rmtree(sites_enabled_dir)
        shutil.copytree(os.path.join(backup_dir, "sites-enabled"), sites_enabled_dir)
        print("Restored previous Nginx configuration.")

# Clean up the backup directory
if cleanup_backup:
    shutil.rmtree(backup_dir)